enum Square {
    Empty,
    Cross,
    Circle,
}

struct Board {
    board: [Square; 9],
}

impl Board {
    /// Creates a new empty board
    fn new() -> Self {
        Self {
            board: [
                Square::Empty,
                Square::Cross,
                Square::Circle,
                Square::Empty,
                Square::Empty,
                Square::Empty,
                Square::Empty,
                Square::Empty,
                Square::Empty,
            ],
        }
    }

    /// Display the board to standard output
    fn display(&self) {
        for row in 0..3 {
            println!(
                " {} | {} | {}",
                square_to_char(&self.board[row * 3]),
                square_to_char(&self.board[row * 3 + 1]),
                square_to_char(&self.board[row * 3 + 2])
            );
            if row == 0 || row == 1 {
                println!("---+---+---");
            }
        }
    }
}

fn square_to_char(s: &Square) -> char {
    match s {
        Square::Empty => '.',
        Square::Cross => 'X',
        Square::Circle => 'O',
    }
}

fn main() {
    let board = Board::new();

    board.display();
}

// Will only be build in 'test' configuration
#[cfg(test)]
// A module where the tests are located, more on modules later
mod tests {
    use super::*;
    
    #[test]
    fn should_pass() {
        assert!(true);
    }
    #[test]
    #[should_panic]
    fn should_panic() {
        panic!("Doing wrong things on purpose here!");
    }

    #[test]
    fn new_board_squares_are_empty() {
        let board = Board::new();

        for square in board.board {

            // Cannot do that due to 'trait' issues, more on that
            // later
            //            assert_eq!(square, Square::Empty);
            match square {
                Square::Empty => (),
                Square::Cross | Square::Circle => panic!("Square is not empty"),
            }
        }
    }
}
